using System;
using System.Runtime.InteropServices;
using SDL2;
using SlatedGameToolkit.Framework.Exceptions;
using SlatedGameToolkit.Framework.Graphics.Window;

namespace SlatedGameToolkit.Framework.Graphics.Programs
{
    public abstract class GLProgram : IDisposable
    {
        private WindowContext context;
        private protected readonly UIntPtr handle;
        private protected GLCreateProgram createProgram;
        private protected GLDeleteProgram deleteProgram;
        private protected GLLinkProgram linkProgram;
        private protected GLGetProgramInfoLog getProgramInfoLog;
        private protected GLProgramParameter programParameter;
        private protected GLUseProgram useProgram;

        /// <summary>
        /// Creates an OpenGL program.
        /// 
        /// This is bound to the current context.
        /// </summary>
        internal GLProgram() {
            this.context = WindowContextsManager.CurrentWindowContext;
            createProgram = GLFunctionUtils.RetrieveGLDelegate<GLCreateProgram>("glCreateProgram");
            deleteProgram = GLFunctionUtils.RetrieveGLDelegate<GLDeleteProgram>("glDeleteProgram");
            linkProgram = GLFunctionUtils.RetrieveGLDelegate<GLLinkProgram>("glLinkProgram");
            getProgramInfoLog = GLFunctionUtils.RetrieveGLDelegate<GLGetProgramInfoLog>("glGetProgramInfoLog");
            programParameter = GLFunctionUtils.RetrieveGLDelegate<GLProgramParameter>("glProgramParameteri");
            useProgram = GLFunctionUtils.RetrieveGLDelegate<GLUseProgram>("glUseProgram");

            handle = createProgram();
        }

        /// <summary>
        /// Binds this program.
        /// </summary>
        public virtual void Use() {
            useProgram(handle);
        }

        /// <summary>
        /// Disposes of the program.
        /// </summary>
        public virtual void Dispose()
        {
            deleteProgram(handle);
            OpenGLErrorException.CheckGLErrorStatus();
        }

        ~GLProgram() {
            Dispose();
        }
    }
}