using System;
using System.Drawing;
using SlatedGameToolkit.Framework.Graphics.Textures;

namespace SlatedGameToolkit.Framework.Graphics.Render
{
    public class Renderer : IDisposable
    {
        private Batch batch;
        private Texture currentTexture;

        public Renderer(Batch batch) {
            this.batch = batch;
        }

        public Renderer() {
            this.batch = new Batch();
        }

        public void Dispose()
        {
            batch.Dispose();
        }

        public void Queue(IRenderable renderable, Color color) {
            if (renderable.Texture != currentTexture) {
                if (batch.Batching) batch.End();
                currentTexture = renderable.Texture;
                currentTexture.Use();
                batch.Begin(currentTexture);
            }
            batch.Add(renderable, color);
        }

        public void Render() {
            if (batch.Batching) batch.End();
        }
    }
}