using System;
using System.Runtime.InteropServices;
using SDL2;
using SlatedGameToolkit.Framework.Exceptions;
using SlatedGameToolkit.Framework.Graphics.Window;

namespace SlatedGameToolkit.Framework.Graphics.Shaders
{
    public abstract class GLShader : IDisposable {
        private WindowContext context;
        public UIntPtr Handle { get; private protected set; }

        private protected GLCreateShader createShader;
        private protected GLShaderSource shaderSource;
        private protected GLCompileShader compileShader;
        private protected GLGetShaderLogInfo getShaderLogInfo;
        private protected GLGetAttribLocation getAttribLocation;
        private GLDeleteShader deleteShader;

        internal GLShader() {
            context = WindowContextsManager.CurrentWindowContext;
            createShader = GLFunctionUtils.RetrieveGLDelegate<GLCreateShader>("glCreateShader");
            shaderSource = GLFunctionUtils.RetrieveGLDelegate<GLShaderSource>("glShaderSource");
            compileShader = GLFunctionUtils.RetrieveGLDelegate<GLCompileShader>("glCompileShader");
            getShaderLogInfo = GLFunctionUtils.RetrieveGLDelegate<GLGetShaderLogInfo>("glGetShaderLogInfo");
            deleteShader = GLFunctionUtils.RetrieveGLDelegate<GLDeleteShader>("glDeleteShader");
            getAttribLocation = GLFunctionUtils.RetrieveGLDelegate<GLGetAttribLocation>("glGetAttribLocation");
        }

        /// <summary>
        /// Gets the attribute location.
        /// </summary>
        /// <param name="attributeName">The name of the attribute.</param>
        /// <returns>The attribute location index.</returns>
        public int GetAttributeLocation(string attributeName) {
            int index = getAttribLocation(Handle, attributeName);
            OpenGLErrorException.CheckGLErrorStatus();
            return index;
        }
        
        /// <summary>
        /// Disposes of the shader at the handle.
        /// </summary>
        public virtual void Dispose()
        {
            deleteShader(Handle);
            OpenGLErrorException.CheckGLErrorStatus();
        }

        ~GLShader() {
            Dispose();
        }
    }
}