using System;

namespace SlatedGameToolkit.Framework.Graphics.Shaders
{
    public class VertexArray : IDisposable
    {
        private GLVertexArraySet glVertexArraySet;
        private uint index;

        public VertexArray(GLVertexArraySet gLVertexArraySet, uint index) {
            this.glVertexArraySet = gLVertexArraySet;
            this.index = index;
        }

        public VertexArray() {
            this.glVertexArraySet = new GLVertexArraySet(1);
            this.index = 0;
        }

        public void BufferVertices(float[] data, bool dynamic = true) {
            glVertexArraySet.BufferVertices(index, data, dynamic);
        }

        public void BufferIndices(uint[] data, bool dynamic = true) {
            glVertexArraySet.BufferIndices(index, data, dynamic);
        }

        /// <summary>
        /// Defines the vertex attributes in an OpenGL vertex array object.
        /// Sends as floats.
        /// </summary>
        /// <param name="enableAttributes">Whether or not to automatically enable the attributes that are being defined. Defaults to true.</param>
        /// <param name="definitions">The definitions for the vertex array object.</param>
        public void defineVertexAttributes(bool enableAttributes = true, params VertexAttributeDefinition[] definitions) {
            this.glVertexArraySet.defineVertexAttributes(index, enableAttributes, definitions);
        }

        /// <summary>
        /// Enaables the vertex array object's definitions at the given attribute indices.
        /// </summary>
        /// <param name="attributeIndices">The attribute indices to be enabled.</param>
        public void EnableAttributes(params uint[] attributeIndices) {
            glVertexArraySet.EnableAttributes(index, attributeIndices);
        }

        /// <summary>
        /// Disables the vertex array object's definitions at the given attribute indices.
        /// </summary>
        /// <param name="attributeIndices">The attribute indices to be disabled.</param>
        public void DisableAttributes(params uint[] attributeIndices) {
            glVertexArraySet.DisableAttributes(index, attributeIndices);
        }

        public void Use() {
            glVertexArraySet.Use(index);
        }

        public void Dispose()
        {
            this.glVertexArraySet.Dispose();
        }
    }
}