using System;
using SlatedGameToolkit.Framework.Exceptions;
using SlatedGameToolkit.Framework.Graphics;
using SlatedGameToolkit.Framework.Graphics.Textures;

namespace SlatedGameToolkit.Framework.Graphics
{
    public class GLTexture2DSet : IDisposable
    {
        private bool disposed;
        public int Count { get; private set; }
        private GLGenTextures genTextures;
        private GLTexParameteri texParameteri;
        private GLTexParameterfv texParameterfv;
        private GLDeleteTextures deleteTextures;
        private GLTexImage2D texImage2D;
        private GLBindTexture bindTexture;
        private GLGenerateMipMap generateMipMap;
        private readonly UIntPtr[] handle;
        private readonly bool linear, mipmap;

        public GLTexture2DSet(TextureData[] textureDatas, bool linear = false, bool mipmap = false)
        {
            this.linear = linear;
            this.mipmap = mipmap;
            genTextures = GLFunctionUtils.RetrieveGLDelegate<GLGenTextures>("glGenTextures");
            deleteTextures = GLFunctionUtils.RetrieveGLDelegate<GLDeleteTextures>("glDeleteTextures");
            bindTexture = GLFunctionUtils.RetrieveGLDelegate<GLBindTexture>("glBindTexture");
            texParameteri = GLFunctionUtils.RetrieveGLDelegate<GLTexParameteri>("glTexParameteri");
            texParameterfv = GLFunctionUtils.RetrieveGLDelegate<GLTexParameterfv>("glTexParameterfv");
            texImage2D = GLFunctionUtils.RetrieveGLDelegate<GLTexImage2D>("glTexImage2D");
            generateMipMap = GLFunctionUtils.RetrieveGLDelegate<GLGenerateMipMap>("glGenerateMipMap");
            handle = new UIntPtr[textureDatas.Length];
            genTextures((uint)textureDatas.Length, handle);
            for (uint i = 0; i < textureDatas.Length; i++) {
                Setup(i, textureDatas[i]);
            }
            bindTexture(GLEnums.GL_TEXTURE_2D, UIntPtr.Zero);
            OpenGLErrorException.CheckGLErrorStatus();

            this.Count = textureDatas.Length;
        }

        private void Setup(uint index, TextureData textureData) {
            bindTexture(GLEnums.GL_TEXTURE_2D, handle[index]);
            texParameteri(GLEnums.GL_TEXTURE_2D, GLEnums.GL_TEXTURE_WRAP_S, (int)GLEnums.GL_CLAMP_TO_EDGE);
            texParameteri(GLEnums.GL_TEXTURE_2D, GLEnums.GL_TEXTURE_WRAP_S, (int)GLEnums.GL_CLAMP_TO_EDGE);
            texParameteri(GLEnums.GL_TEXTURE_2D, GLEnums.GL_TEXTURE_MIN_FILTER, (int)(linear ? (mipmap ? GLEnums.GL_LINEAR_MIPMAP_LINEAR : GLEnums.GL_LINEAR) : (mipmap ? GLEnums.GL_NEAREST_MIPMAP_LINEAR : GLEnums.GL_NEAREST)));
            texParameteri(GLEnums.GL_TEXTURE_2D, GLEnums.GL_TEXTURE_MAG_FILTER, (int)(linear ? GLEnums.GL_LINEAR : GLEnums.GL_NEAREST));
            texImage2D(GLEnums.GL_TEXTURE_2D, 0, (int)GLEnums.GL_RGBA, textureData.width, textureData.height, 0, GLEnums.GL_RGBA, GLEnums.GL_UNSIGNED_BYTE, textureData.data);
            generateMipMap(GLEnums.GL_TEXTURE_2D);
        }

        public void Bind(uint index) {
            bindTexture(GLEnums.GL_TEXTURE_2D, handle[index]);

        }

        public void Dispose()
        {
            if (disposed) return;
            this.disposed = true;
            deleteTextures((uint)handle.Length, handle);
            OpenGLErrorException.CheckGLErrorStatus();
            GC.SuppressFinalize(this);
        }
    }
}