using System.Collections.Generic;

namespace SlatedGameToolkit.Framework.Graphics.Window
{
    public static class WindowContextsManager {
        private static WindowContext current;

        /// <summary>
        /// The current window context.
        /// </summary>
        /// <value>The currently active window context.</value>
        public static WindowContext CurrentWindowContext {
            get {
                return current;
            }

            set {
                if (current != value) {
                    current = value;
                    value.MakeCurrent();
                }
            }
        }
        private static Dictionary<uint, WindowContext> existingWindows = new Dictionary<uint, WindowContext>();

        internal static void RegisterWindow(WindowContext windowHandle) {
            GameEngine.Logger.Debug("Registering window: " + windowHandle.WindowID);
            existingWindows.Add(windowHandle.WindowID, windowHandle);
        }
        
        internal static void DeregisterWindow(WindowContext windowHandle) {
            GameEngine.Logger.Debug("Deregistering window: " + windowHandle.WindowID);
            existingWindows.Remove(windowHandle.WindowID);
        }

        public static WindowContext ContextFromWindowID(uint ID) {
            return existingWindows[ID];
        }
    }
}